#include <string.h>

#include "pam_ru.h"
#include "ru_grp.h"

/*
* nss_check - 
*
* return 0 if agent is allowed to become target without password
*       -1 otherwise
* no obvious side effects, but does do setgrent and endgrent
*/

/*
 * Use the target to recover the group ID of the become account.  Then,
 * check the group ID of all group entries.  Those that match the become
 * account group ID AND have the group password "become" contain members
 * of the become account.  Finally, check among those users to see if the
 * agent is allowed access to the become account.
 */

int nss_check(char *target, char *agent, int debug)
{
    
   struct group *search_grent;
   char **gmem;                /* Vector of group entries              */
   int flag; /* flag to see if the user is found in the become group */
   struct group grent;
   char grbuf[4096];
   gid_t search_gid;
    
   setgrent();
   
   flag = 0;
    
   /* 
    * look at entries containing "become" in the passwd field, with
    * the user we're trying to change to as the first user in the list.
    * If we're anywhere in the list, it's OK.  Note that there can in
    * theory be more than one group controlling the same user. I'm not
    * sure why you'd want to do that.
    */
    
    if (getgrnam_ru(target, &grent, grbuf, sizeof(grbuf)) == 0) { 
       if (strcmp(grent.gr_passwd, "become") == 0) {
	    if (debug) debout("nss_check: group found");
	    search_gid = grent.gr_gid;
	    
	    if (debug) debout("nss_check: search_gid = %i", search_gid);
	    
	    /* work around a limitation in Sun's NIS which uses ndbm to store
	    NIS info so that long group member lists need to be split.
	    find all groups with the same gid and (effectively)
	    concatenate the member lists.  i.e.:
	    group1:become:12345:user1,user2,user3....
	    group2:become:12345:usera,userb,userc....
	    so in this case, both group1 and group2 have the same gid, 
	    and therefore should be treated as the same become group. 
             */
	    
	    while (!getgrent_ru(&grent, grbuf, sizeof(grbuf), &search_grent)) {			    
		gmem = search_grent->gr_mem;     
		
		if (!strcmp(search_grent->gr_passwd, "become") && 
		    	*gmem && search_gid == search_grent->gr_gid) 
		{
		    if(debug) debout("nss_check: found proper become group");
		    while( *gmem && !flag ) {
			if(!strcmp( *gmem++, agent)) { 
			    flag++;
			}
		    }
		    if (flag) break;
		}
	    }
	    
	    if (debug) debout("nss_check: out of getgrent_ru while loop");
	    
	} else { 
	    if (debug) debout("nss_check: target not a become account"); 
	}
    } else { 
	if(debug) debout("nss_check: Cannot find the become group."); 
    }
    
    endgrent();
    
    if (debug) debout("nss_check: flag = %i", flag);
    return flag;
}

/*
 * nss_is_become - 
 *
 * return 1 if target is a become account
 *        0 otherwise
 */

int nss_is_become(char *target, int debug)
{   
    int flag; /* flag to see if the target is a become account */
    struct group grent;
    char grbuf[4096];

    flag = 0;

    /* 
     * Check to see if target's passwd entry contains "become". 
     * If so, target is a become account.
     */
    if (getgrnam_ru(target, &grent, grbuf, sizeof(grbuf)) == 0) {
        if(debug) debout("nss_is_become: Target group found.");
        if (strcmp(grent.gr_passwd, "become") == 0) {
            if(debug) debout("nss_is_become: Target is a become account.");
            flag = 1;
        }
    } else {
        if(debug) debout("nss_is_become: Cannot find target group");
    }

    return flag;
}

